#ifndef _PRPARTEDITOR_H_
#define _PRPARTEDITOR_H_

#include "atom.h"
#include "position.h"


class Table;
class Note;
class Part;
class Track;
class Reference;
class Event;

class PrPartEditor
{
 protected:
  Table *   _parts;       // An entry is always the first part in a track, so the number of parts is the number of displayed tracks
  Reference * _current; // pointing to the part currently edited
  Position  _leftPos;    // left pos
  Position  _rightPos;   // (obsolete: caution: this has to be updated manualy, e.g. in rewind())
  int       _current_length;  // current length value (192 = 1/8, etc)
  int       _current_enh;
  int       _current_vel;
  int       _current_dot;
  bool      _current_tri;
  char *    _cLeft;
  char *    _cRight;
  int       _bars;
  int       _grid;
  bool      _color;
  bool      _speaker;

 public:
  PrPartEditor();
  PrPartEditor(Part * p);
  PrPartEditor(Track * t);
  ~PrPartEditor();

  /** returns the part currently edited
   */
  Part * part();

  /** returns the part i
   */
  Part * part(int i);

  /** sets the current part
   */
  Part * setPart(int i);

  /** returns the number of parts edited by this editor
   */
  int parts();

  /** returns the name of the part currently edited
   */
  char * name();

  /** moves one track down and returns the current part
   */
  Part * moveDown();

  /** moves one track up and returns the current part
   */
  Part * moveUp();

  /** returns the left position as char
   */
  char * cLeft();

  /** returns the right position as char
   */
  char * cRight();
  
  /** returns the left position as Position
   */
  Position left();

  /** returns the right position as Position
   */
  Position right();
  
  /** moves the positions one bar to the left
   */
  void moveleft(int i=1);

  /** moves the positions one bar to the right
   */
  void moveright(int i=1);

  /** moves the positions to the left position.
   * Unless the part is specified, the currently selected part is used
   */
  void rewind(Part * part=0);

  /** moves the positions to the specified position offset.
   */
  void wind(Position p);

  /** moves the positions to the specified bar offset
   */
  void wind(int bar);

  /** always call rewind() before using currentInc(). This calls the part's currentInc() method.
   * Unless the part is specified, the currently selected part is used
   */
  bool currentInc(Part * pt=0, long ticks=0);

  /** zooms in
   */
  void zoomin();

  /** zooms out
   */
  void zoomout();

  /** returns the current length value in ticks
   */
  int length();

  /** returns the current length value in ticks, including increase through dots.
   */
  int totalLength();

  /** returns the current length value as an ordered int (1, ..., 7)
   */
  int lengthOrd();

  /** sets the length value as ticks
   */
  void setLength(int);

  /** sets the length value as an ordered int (1, ..., 7)
   */
  void setLengthOrd(int);

  /** returns the number of dots
   */
  int dot();

  /** sets the number of dots
   */
  void setDot(int);

  /** returns true in triplet mode
   */
  bool triplet();

  /** sets the triplet mode
   */
  void setTriplet(bool);

  /** returns the enharmonic shift value (-2=bb, -1=b, 0=nat, 1=#, 2=x).
   */
  int enharmonicShift();

  /** sets the enharmonic shift value (-2=bb, -1=b, 0=nat, 1=#, 2=x).
   */
  void setEnharmonicShift(int);

  int velocity();

  void setVelocity(int);

  /** implement this for your own editor
   */
  virtual void update()=0;

  /** if a note exists at the specified pitch and position (up to the note's end) this note is returned, or zero else
   */
  Note * getGrabNote(long pos, int pitch, bool & tail);
  
  /** the specified position is quantized and returned
   */
  long snap(long pos);

  int grid();
  
  void setGrid(int);

  bool color();

  bool speaker();

  void setColor(bool);
  
  void setSpeaker(bool);
  
  /** if the selection contains exactly one event, it is returned by this method. Null otherwise
   */
  Event * currentEvent();

  char ** actionList();

  /** enables and disables the gui buttons for the GuiEvents: changes, undo, redo, selection and memory
   */
  virtual void gui(GuiEvents ev, bool flag)=0;

  /** prints the editor's content
   */
  virtual void print()=0;

  /** the editor's settings
   */
  virtual void settings()=0;

};


#endif

